from pydantic import BaseModel
import os
from pathlib import Path


class IngestFolder:
    def __init__(self, repo, ingest_file, allowed_extensions, logger):
        self.repo = repo
        self.ingest_file = ingest_file
        self.allowed_extensions = allowed_extensions
        self.logger = logger if logger else print

    class Inputs(BaseModel):
        folder: Path
        dataset: str = None
        user: dict

    class Outputs(BaseModel):
        dataset: dict

    def __call__(self, inputs: Inputs) -> Outputs:
        self.logger("Uploading directory (only files, not recursive)")
        items = list(inputs.folder.glob("*"))
        filtered_items = [item for item in items if item.is_file()]
        filtered_items = [
            item for item in filtered_items if item.suffix in self.allowed_extensions
        ]
        if len(filtered_items) == 0:
            raise Exception("No files found in directory")
        if len(filtered_items) > 10:
            raise Exception("Too many files in directory, limited to 10")
        self.logger("The following files will be uploaded:")
        for item in filtered_items:
            self.logger(f"{item.name}")
        for item in filtered_items:
            data = self.ingest_file(item, inputs.dataset, logger=self.logger)
        return self.Outputs(dataset=data)



import typer
from pathlib import Path

from ..datasets import (
    retrieve_datasets,
    download_dataset,
    ingest_file,
    ingest_folder,
)

app = typer.Typer()


@app.command()
def ingest(path: Path, dataset: str):
    """
    Ingest a file

    path: Path to folder with data (limited to 10 files, not recursive!)
    dataset: Name of the dataset
    """
    try:
        if path.is_dir():
            ingest_folder(path, dataset, typer.echo)
        else:
            ingest_file(path, dataset, typer.echo)
    except Exception as e:
        typer.echo(e)


@app.command()
def list():
    """
    List all datasets and files
    """
    datasets = retrieve_datasets()
    typer.echo(datasets)


@app.command()
def get(dataset: str, file: str = None, path: str = None):
    """
    Download a dataset

    dataset: Name of the dataset
    file: Name of the file to download (optional, if not provided, the whole dataset will be downloaded)
    path: Path to download the dataset to (optional, if not provided, the dataset will be downloaded to ~/.eotdl/datasets)
    """
    try:
        dst_path = download_dataset(dataset, file, path, typer.echo)
        typer.echo(f"Data available at {dst_path}")
    except Exception as e:
        typer.echo(e)


if __name__ == "__main__":
    app()



from .DownloadDataset import DownloadDataset
from .IngestDataset import IngestDataset
from .IngestLargeDataset import IngestLargeDataset
from .RetrieveDataset import RetrieveDataset
from .RetrieveDatasets import RetrieveDatasets
from .IngestFile import IngestFile
from .IngestFolder import IngestFolder



from pydantic import BaseModel
from ....src.utils import calculate_checksum


class DownloadDataset:
    def __init__(self, repo, retrieve_dataset, logger):
        self.repo = repo
        self.retrieve_dataset = retrieve_dataset
        self.logger = logger if logger else print

    class Inputs(BaseModel):
        dataset: str
        file: str = None
        path: str = None
        user: dict

    class Outputs(BaseModel):
        dst_path: str

    def download(self, dataset, dataset_id, file, checksum, path, user):
        self.logger(f"Downloading {file}")
        dst_path = self.repo.download_file(
            dataset, dataset_id, file, user["id_token"], path
        )
        if calculate_checksum(dst_path) != checksum:
            self.logger(f"Checksum for {file} does not match")
        self.logger(f"Done")
        return dst_path

    def __call__(self, inputs: Inputs) -> Outputs:
        dataset = self.retrieve_dataset(inputs.dataset)
        if inputs.file:
            files = [f for f in dataset["files"] if f["name"] == inputs.file]
            if not files:
                raise Exception(f"File {inputs.file} not found")
            if len(files) > 1:
                raise Exception(f"Multiple files with name {inputs.file} found")
            dst_path = self.download(
                inputs.dataset,
                dataset["id"],
                inputs.file,
                files[0]["checksum"],
                inputs.path,
                inputs.user,
            )
            return self.Outputs(dst_path=dst_path)
        for file in dataset["files"]:
            dst_path = self.download(
                inputs.dataset,
                dataset["id"],
                file["name"],
                file["checksum"],
                inputs.path,
                inputs.user,
            )
        return self.Outputs(dst_path="/".join(dst_path.split("/")[:-1]))



from ..src.repos import APIRepo
from ..src.usecases.datasets import IngestFile, IngestFolder
from ..auth import with_auth

allowed_extensions = [
    ".zip",
    ".tar",
    ".tar.gz",
    ".csv",
    ".txt",
    ".json",
    ".pdf",
    ".md",
]


@with_auth
def ingest_file(file, dataset, logger=None, user=None):
    api_repo = APIRepo()
    ingest = IngestFile(api_repo, allowed_extensions, logger)
    inputs = ingest.Inputs(file=file, dataset=dataset, user=user)
    outputs = ingest(inputs)
    return outputs.dataset


@with_auth
def ingest_folder(folder, dataset, logger=None, user=None):
    api_repo = APIRepo()
    ingest = IngestFolder(api_repo, ingest_file, allowed_extensions, logger)
    inputs = ingest.Inputs(folder=folder, dataset=dataset, user=user)
    outputs = ingest(inputs)
    return outputs.dataset


# @with_auth
# def ingest_dataset(name, description, path, logger=None, user=None):
#     api_repo = APIRepo()
#     ingest = IngestDataset(
#         api_repo,
#     )
#     inputs = ingest.Inputs(name=name, description=description, path=path, user=user)
#     outputs = ingest(inputs)
#     return outputs.dataset


# @with_auth
# def ingest_large_dataset(name, path, logger=None, user=None):
#     api_repo = APIRepo()
#     ingest = IngestLargeDataset(api_repo, logger)
#     inputs = ingest.Inputs(name=name, path=path, user=user)
#     outputs = ingest(inputs)
#     return outputs.dataset


# def ingest_q0(dataset, path):
#     return ingest_large_dataset(dataset, path)


# def ingest_q1(dataset, stac_catalog):
#     print("holas")
#     return
